# $Id: List.pm 33876 2011-02-14 14:06:57Z wsl $
# $URL: https://svn.uvt.nl/its-id/trunk/sources/squarepeg/lib/UvT/Squarepeg/Ajax/User/Boxes/List.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package UvT::Squarepeg::Ajax::User::Boxes::List::Document;

use Xyzzy::Document -self;

sub build {
	my $req = $self->req;

	$self->setDocumentElement($self->construct('ok', map {
		my ($box, $name, $quota, $usage, $manager) = @$_;
		[mbox => [box => $box], [name => $name], [quota => $quota], [usage => $usage // 0], [manager => $manager ? 'true' : 'false']]
	} @{$req->boxes}));
}

package UvT::Squarepeg::Ajax::User::Boxes::List::Request;

use UvT::Squarepeg::Ajax::User::Request -self;

field boxes => sub {
	my $self = shift;

	my $uid = $self->uid;

	die $self->errorpage('permission_denied')
		unless $self->is_admin_for_user($uid);

	my $db = $self->db;
	my $q = $db->prepare_cached('SELECT mailboxes.box, mailboxes.name, mailboxes.quota, maillinks.manager
		FROM mailnames JOIN maillinks USING (mailuser) JOIN mailboxes USING (mailbox)
		WHERE mailnames.uid = ? ORDER BY mailboxes.box');
	$q->execute($uid);
	my $res = $q->fetchall_arrayref;
	$q->finish;

	$db->commit;

	my $quota = $self->cfg->quota;

	my @res = map {
			my ($box, $name, $limit, $manager) = @$_;
			my $usage = $quota->get($box);
			[$box, $name, $limit, $usage, $manager]
		} @$res;

	return \@res;
};

package UvT::Squarepeg::Ajax::User::Boxes::List;

use UvT::Squarepeg::Handler -self;

sub handle {
	my $req = new UvT::Squarepeg::Ajax::User::Boxes::List::Request(cfg => $self, ctx => shift);
	my $doc = new UvT::Squarepeg::Ajax::User::Boxes::List::Document(req => $req);
	return $doc->response;
}
